#include <stdio.h>
#include <string.h>
#include <ctype.h>

typedef struct MenuePunkt_s MenuePunkt_t;
struct MenuePunkt_s
{  char  zeichen; // Auswahl: Kleinbuchstabe od. Zeichen
   char *text;    // Informeller Text
};
// Auflistung aller Menuepunkte
static MenuePunkt_t menuePunkte[] =
{  { '1', "Eingabe" },
   { '2', "Ausgabe" },
   { '3', "Suchen"  },
   { '4', "Lschen" },
   { '5', "Laden" },
   { '6', "Speichern" },
   { '0', "Beenden" },
   { 'm', "Menue"   },
   { 0 }  // Endemarkierung ist 0
};

#define DATEINAME_LEN 50
#define ADRESSBUCH_LEN 20

#define NAME_LEN      30
#define ORT_LEN       20
#define STRASSE_LEN   50

typedef struct Adresse_s
{  char name[NAME_LEN];       // Name der Person
   long plz;                  // Postleitzahl
   char ort[ORT_LEN];         // Wohnort
   char strasse[STRASSE_LEN]; // Strasse
   long nummer;               // Hausnummer
} Adresse_t;

/* LeereEingabepuffer: Leert den Standardeingabepuffer
 *                     bis zum nchsten '\n'
 * Parameter:    Keiner
 * Rckgabewert: Keiner
 */
void LeereEingabepuffer()
{  while (getchar() != '\n');
}  // end LeereEingabepuffer

/* Menue: Gibt ein Menue aus
 *        Verwendet Strukturen aus menuePunkte
 * Parameter:    Keiner
 * Rckgabewert: Keiner
 */
void Menue()
{  MenuePunkt_t *mp;

   printf("\n");
   printf("Men\n");
   printf("=====\n");

   for (mp = menuePunkte; mp->zeichen; mp = mp + 1)
      printf(" %c ... %s\n", mp->zeichen, mp->text);

   printf ("\n");
}  // end Menue

/* MenueAuswahl: Benutzerabfrage zum Menue
 * Parameter:    Keiner
 * Rckgabewert: Korrekte Benutzereingabe
 *          Typ: char
 */
char MenueAuswahl()
{  MenuePunkt_t *mp;
   char zeichen;

   while (1)
   {  printf("Wahl (m fr Menue): ");
      zeichen = getchar();
      zeichen = tolower(zeichen);
      LeereEingabepuffer();

      if (zeichen == 'm')
      {  Menue();
         continue;
      }
      for (mp = menuePunkte; mp->zeichen; mp = mp + 1)
         if (zeichen == mp->zeichen)
            return zeichen;
      fprintf(stderr, "Falsche Eingabe!\n");
   }
}  // end MenueAuswahl

// Fr das Men werden die Funktionen aus dem vorherigen
// Beispiel gebraucht!

/* getline: Liest eine Textzeile ein
 * Parameter:
 *     str: Zeichenfeld
 *     num: Lnge von Zeichenfeld
 * Rckgabewert: Keiner
 */
void getline (char *str, long num)
{  char c, *strende = str + num - 1;

   if (num < 1 || !str)
      return;

   while ((c = getchar ()) != '\n')
      if (str < strende)
      {  *str = c;
         str = str + 1;
      }

   *str = 0;
}  // end getline

/* Eingabe: Einlesen eines Datensatzes
 * Parameter:
 *     adressen: Die gesamte Datenbank
 *     index   : Index des neuen Elementes
 *               gleichzeitig Lnge des Feldes
 * Rckgabewert: Keiner
 */
void Eingabe(Adresse_t adressen[], long *index)
{  printf("\n");
   printf("Name:       ");
   getline(adressen[*index].name, sizeof (adressen[*index].name));
   printf("PLZ:        ");
   scanf("%ld", &adressen[*index].plz);
   getchar();
   printf("Ort:        ");
   getline(adressen[*index].ort, sizeof(adressen[*index].ort));
   printf("Strae:     ");
   getline(adressen[*index].strasse, sizeof(adressen[*index].strasse));
   printf("Hausnummer: ");
   scanf("%ld", &adressen[*index].nummer);
   getchar();
   printf("\n");
   *index = *index + 1;
}  // end Eingabe

/* AusgabeDatensatz: Ausgabe eines Datensatzes
 * Parameter:
 *     adressen: Die gesamte Datenbank
 *     index   : Index des Elementes
 * Rckgabewert: Keiner
 */
void AusgabeDatensatz(Adresse_t adressen[], long index)
{  printf("\n");
   printf("Name:       %s\n",  adressen[index].name);
   printf("PLZ:        %ld\n", adressen[index].plz);
   printf("Ort:        %s\n",  adressen[index].ort);
   printf("Strae:     %s\n",  adressen[index].strasse);
   printf("Hausnummer: %ld\n", adressen[index].nummer);
}  // end AusgabeDatensatz

/* Ausgabe: Ausgabe aller Datenstze
 * Parameter:
 *     adressen: Die gesamte Datenbank
 *     anzahl  : Lnge des Feldes
 * Rckgabewert: Keiner
 */
void Ausgabe(Adresse_t adressen[], long anzahl)
{  long i;

   for (i = 0; i < anzahl; i = i + 1)
      AusgabeDatensatz(adressen, i);

   printf("\n");
}  // end Ausgabe

/* SucheName: Suchen nach einem Datensatz anhand
 *            des Namens
 * Parameter:
 *     adressen: Die gesamte Datenbank
 *     anzahl  : Lnge des Feldes
 * Rckgabewert: Position im Feld. -1 bei Fehler
 *          Typ: long
 */
long SucheName(Adresse_t adressen[], long anzahl)
{  char name[NAME_LEN];
   long i;

   printf("Name: ");
   getline(name, NAME_LEN);

   for (i = 0; i < anzahl; i = i + 1)
      if (!(strncmp(name, adressen[i].name, strlen(name))))
         return i;

   return -1;
}  // end SucheName

/* LoescheDatensatz: Lschen eines Datensatzes anhand des Namens
 * Parameter:
 *     adressen: Die gesamte Datenbank
 *     anzahl  : Lnge des Feldes
 * Rckgabewert: Keiner
 */
void LoescheDatensatz(Adresse_t adressen[], long *anzahl)
{  long index;

   index = SucheName(adressen, *anzahl);

   if (index == -1)
   {  printf("\n");
      fprintf(stderr, "Datensatz nicht gefunden!\n");
      printf("\n");
      return;
   }

   printf("\n");
   AusgabeDatensatz(adressen, index);
   printf("\n");

   printf("Datensatz lschen?: ");
   {  char zeichen;

      zeichen = getchar();
      getchar();
      if (tolower(zeichen) != 'j')
         return;
   }

   for (index = index + 1; index < *anzahl; index = index + 1)
      adressen[index - 1] = adressen[index];

   *anzahl = *anzahl - 1;
}  // end LoescheDatensatz

/* Speichern: Speichert die Datenbank
 * Parameter:
 *     adressen: Die gesamte Datenbank
 *     anzahl  : Lnge des Feldes
 * Rckgabewert: 0 bei Fehler, sonst OK
 *          Typ: long
 */
long Speichern(Adresse_t adressen[], long anzahl)
{  char  dateiname [DATEINAME_LEN];
   FILE *datei;

   printf("Dateiname: ");
   scanf("%s", dateiname);
   getchar();

   datei = fopen(dateiname, "w");
   if (!datei)
   {  fprintf(stderr, "Datei \"%s\" kann nicht gefunden werden\n",
              dateiname);
      return 0;
   }

   // Speichern aller Datenstze
   if (fwrite(adressen, sizeof(Adresse_t), anzahl, datei) < anzahl)
   {  fprintf(stderr, "Datei \"%s\" kann nicht gefunden werden\n",
              dateiname);
      return 0;
   }

   // Schlieen der Dateien
   fclose(datei);

   return 1;
}  // end Speichern

/* Laden: Ldt die Datenbank
 * Parameter:
 *     adressen: Die gesamte Datenbank
 *     anzahl  : Lnge des Feldes
 * Rckgabewert: 0 bei Fehler, sonst OK
 *          Typ: long
 */
long Laden(Adresse_t adressen[], long *anzahl)
{  char  dateiname [DATEINAME_LEN];
   long  index;
   FILE *datei;

   printf("Dateiname: ");
   scanf("%s", dateiname);
   getchar();

   datei = fopen(dateiname, "r");
   if (!datei)
   {  fprintf(stderr, "Datei \"%s\" kann nicht gefunden werden\n",
              dateiname);
      return 0;
   }

   // Laden der Datenstze
   for (index = 0; 1; index = index + 1)
      if (!fread(&adressen[index], sizeof(Adresse_t), 1, datei))
         break;

   // Merken der Anzahl der gelesenen Datenstze
   *anzahl = index;

   // Schlieen der Dateien
   fclose(datei);

   return 1;
}  // end Laden

main()
{  char zeichen;
   Adresse_t Adressbuch[ADRESSBUCH_LEN];
   long anzahl;

   printf("Kleine Adressverwaltung\n");
   printf("=======================\n");

   Menue();

   do
   {  zeichen = MenueAuswahl();
                
      switch (zeichen)
      {case '1':
         if (anzahl < ADRESSBUCH_LEN)
            Eingabe(Adressbuch, &anzahl);
         else
            fprintf(stderr, "Datenbank voll!\n");
         break;
       case '2':
         Ausgabe(Adressbuch, anzahl);
         break;
       case '3':
         {  long index;

            printf("\n");
            index = SucheName(Adressbuch, anzahl);
            if (index == -1)
            {  printf("\n");
               fprintf(stderr, "Datensatz nicht gefunden!\n");
               printf("\n");
            }
            else
            {  AusgabeDatensatz(Adressbuch, index);
               printf("\n");
            }
         }
         break;
       case '4':
         printf("\n");
         LoescheDatensatz(Adressbuch, &anzahl);
         printf("\n");
         break;
       case '5':
         printf("\n");
         Laden(Adressbuch, &anzahl);
         printf("\n");
         break;
       case '6':
         printf("\n");
         Speichern(Adressbuch, anzahl);
         printf("\n");
         break;
      }
   } while (zeichen != '0');
}  // end main
